import json
import boto3
import sys
import os

dynamo_client = boto3.client('dynamodb')
primaryRegion = os.environ['PRIMARY_REGION']
secondaryRegion = os.environ['SECONDARY_REGION']
tableName = os.environ['TABLE_NAME']
primaryRegion_client = boto3.client('lex-runtime',region_name=primaryRegion)
secondaryRegion_client = boto3.client('lex-runtime',region_name=secondaryRegion)

def getCurrentPrimaryRegion():
    result = dynamo_client.get_item(
        TableName=tableName,
        Key={  
            'connectRegion': {'S': primaryRegion}  
        }
    )
    return result['Item']['lexRegion']['S'] 
    
def updateTable(region):
    result = dynamo_client.update_item( 
        TableName= tableName,
        Key={  
            'connectRegion': {'S': primaryRegion } 
        },  
        UpdateExpression='set lexRegion = :region',
        ExpressionAttributeValues={
        ':region': {'S':region}
        }
    )
    
#SEND MESSAGE/PUT SESSION ENV VA
def put_session(botname, botalias, user, region):
    print(region,botname, botalias)
    client = primaryRegion_client
    if region == secondaryRegion:
        client = secondaryRegion_client
    try:
        response = client.put_session(botName=botname, botAlias=botalias, userId=user)
        if (response['ResponseMetadata'] and response['ResponseMetadata']['HTTPStatusCode'] and response['ResponseMetadata']['HTTPStatusCode'] != 200) or (not response['sessionId']):  
            return 501
        else:
            if getCurrentPrimaryRegion != region:
                updateTable(region)
        return 200
    except:
        print('ERROR: {}',sys.exc_info()[0])
        return 501

def send_message(botname, botalias, user, region):
    print(region,botname, botalias)
    client = primaryRegion_client
    if region == secondaryRegion:
        client = secondaryRegion_client
    try:
        message = os.environ['SAMPLE_UTTERANCE']
        expectedOutput = os.environ['EXPECTED_RESPONSE']
        response = client.post_text(botName=botname, botAlias=botalias, userId=user, inputText=message)
        if response['message']!=expectedOutput:
            print('ERROR: Expected_Response=Success, Response_Received='+response['message'])
            return 500
        else:
            if getCurrentPrimaryRegion != region:
                updateTable(region)
            return 200
    except:
        print('ERROR: {}',sys.exc_info()[0])
        return 501

def lambda_handler(event, context):
    print(event)
    botName = os.environ['BOTNAME']
    botAlias = os.environ['BOT_ALIAS']
    testUser = os.environ['TEST_USER']
    testMethod = os.environ['TEST_METHOD']
    if testMethod == 'send_message':
        primaryRegion_response = send_message(botName, botAlias, testUser, primaryRegion)
    else:
        primaryRegion_response = put_session(botName, botAlias, testUser, primaryRegion)
    if primaryRegion_response != 501:
        primaryRegion_client.delete_session(botName=botName, botAlias=botAlias, userId=testUser)
    if primaryRegion_response != 200:
        if testMethod == 'send_message':
            secondaryRegion_response = send_message(botName, botAlias, testUser, secondaryRegion)
        else:
            secondaryRegion_response = put_session(botName, botAlias, testUser, secondaryRegion)
        if secondaryRegion_response != 501:
            secondaryRegion_client.delete_session(botName=botName, botAlias=botAlias, userId=testUser)
        if secondaryRegion_response != 200:
            updateTable('err')
    #deleteSessions(botName, botAlias, testUser)
    return {'statusCode': 200,'body': 'Success'}